package fr.shortcircuit.pattern.chain.test;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.beanutils.BeanPredicate;
import org.apache.commons.collections.Predicate;
import org.apache.commons.collections.functors.EqualPredicate;
import org.apache.commons.collections.functors.InstanceofPredicate;
import org.apache.commons.collections.functors.NotNullPredicate;

import fr.shortcircuit.pattern.chain.node.PredicateProcessorNode;
import fr.shortcircuit.pattern.chain.node.TerminalProcessorNode;
import fr.shortcircuit.pattern.chain.node.definition.IChainNode;
import fr.shortcircuit.pattern.chain.request.Request;
import fr.shortcircuit.pattern.chain.request.definition.IRequest;

/** 
 * @author Dimitri Dean DARSEYNE (D3), 
 * Published by Short-Circuit under Creative Commons (CC) Licensing: 
 * Authorship/Paternity, NO Commercial Use, NO Derivative
 * Please check for more informations:
 * http://creativecommons.org/licenses/by-nc-nd/2.0/
 *
 * Auteur Dimitri Dean DARSEYNE (D3),
 * Publié par Short-Circuit sous license Creative Commons (CC):
 * Paternité, PAS d'Utilisation Commerciale, pas de Dérivés/Modifications
 * Pour plus d'informations, se rendre sur:
 * http://creativecommons.org/licenses/by-nc-nd/2.0/fr/ 
 * 
 * @since Short-Circuit 1999
 */

/**
 * A main class:
 * 
 * - Instanciating the 3 Predicate processing nodes + the conclusion processor node.
 * - Defining the chainInitiator (first node), that will trigger the chain behavioral reaction, when client-sidely called.
 * - Building the various IRequest elements, with different arguments satisfying the several chain possibilities.
 * - Using IChainNode.setNextNode setting up the chain nodes order: 
 *   nodeNotNull -> nodeInstanceOfString -> nodeValueEquals -> nodeTerminal    
 * - Calling the chainProcessing on each IRequest instance.
 * - System.out logging the decorated IRequest.matchingPredicates List, resulting from the chain course.
 * 
 * Results:
 * 
 * - The requestNull, with an unset null "value" property, fails on the first node: a NotNullPredicate.
 * - The requestInteger, with a "value" property set as Integer, passes through the NotNullPredicate but fails on the InstanceofPredicate(String.class)
 * - The requestStringNOK, with a "value" property set as "testNOK" String, passes through the NotNullPredicate and the InstanceofPredicate(String.class) 
 *   but fails on EqualPredicate("D3")
 * - The requestStringOK, with a "value" property set as "D3" String, successes on the NotNullPredicate, the InstanceofPredicate(String.class) and    
 *   the EqualPredicate("D3"), reaching the final nodeTerminal, which outputs a message ending the chain processing.   
 *
 * 
 * Une classe principale:
 * 
 * - Instanciant les 3 noeuds de traitements Prédicats  + le noeud de traitement de conclusion.
 * - Définissant l'initiateur de chaîne (premier noeud), qui va déclencher la réaction comportementale de la chaîne, 
 *   lorsqu'appelée par une partie cliente.
 * - Construisant des éléments variés de type IRequest, avec différents arguments satisfaisant aux diverses possibilités de la chaîne.  
 * - Utilisant IChainNode.setNextNode pour établir l'ordre des noeuds de la chaîne:
 *   nodeNotNull -> nodeInstanceOfString -> nodeValueEquals -> nodeTerminal
 * - Appelant le processing de la chaîne sur chacune des instances de type IRequest.   
 * - Loggant avec System.out la liste IRequest.matchingPredicates décorée, résultant du parcours de la chaîne.  
 * 
 * Résultats:
 * 
 * - L'objet requestNull, avec une propriété "value" null non affectée, échoue sur le premier noeud: un NotNullPredicate.
 * - L'objet requestInteger, avec une propriété "value" définie en Integer, passe au travers du NotNullPredicate,
 *   mais s'arrête sur le InstanceofPredicate(String.class)
 * - L'objet requestStringNOK, avec une propriété "value" à "testNOK", passe au travers des NotNullPredicate et InstanceofPredicate(String.class) 
 *   mais échoue sur le EqualPredicate("D3")
 * - L'objet requestStringOK, avec une propriété "value" à "D3", vient à bout du NotNullPredicate, du InstanceofPredicate(String.class) et    
 *   du EqualPredicate("D3"), atteignant le nodeTerminal final, qui renvoie un message clôturant le traitement de la chaîne.    
 * 
 */
public class Main 
{
	/** The chain primal node.
	 * Le noeud primaire de la chaîne */
	public static IChainNode chainInitiator;
	
	
	public static void main(String args[])
	{
		//Création des noeuds
		IChainNode nodeNotNull				= new PredicateProcessorNode(new BeanPredicate("value", NotNullPredicate.getInstance()));
		IChainNode nodeInstanceOfString		= new PredicateProcessorNode(new BeanPredicate("value", new InstanceofPredicate(String.class)));
		IChainNode nodeValueEquals			= new PredicateProcessorNode(new BeanPredicate("value", new EqualPredicate("D3")));
		IChainNode nodeTerminal				= new TerminalProcessorNode();
		
		//Initialisation du noeud primaire
		chainInitiator		  				= nodeNotNull;
		 
		//Création des objets de requêtes
		Request requestNull					= new Request("requestNull", 		null);
		Request requestInteger				= new Request("requestInteger", 	42);		 
		Request requestStringNOK			= new Request("requestStringNOK", 	"testNOK");
		Request requestStringOK				= new Request("requestStringOK", 	"D3");
		 
		//initialisation de l'ordre de la chaîne 
		nodeNotNull.setNextNode(			nodeInstanceOfString);
		nodeInstanceOfString.setNextNode(	nodeValueEquals);
		nodeValueEquals.setNextNode(		nodeTerminal);
		 
		//lancement des traitements de la chaîne sur chacune des requêtes
		chainProcess(requestNull);
		chainProcess(requestInteger);
		chainProcess(requestStringNOK);
		chainProcess(requestStringOK);
	}

	/** Launch the chain on the given IRequest parameter.
	 * Démarre la chaîne sur le paramètre IRequest donné. */
	private static void chainProcess(IRequest request)
	{
		chainInitiator.process(request);
		 
		logMatchedPredicates(request);
	}
	 
	/** Reveals the result of the decoration process, done by the chain on the "matchingPredicates" List property   
	 * Révèle le résultat du processus de décoration, effectué par la chaîne sur la propriété "matchingPredicates" de type List */
	private static void logMatchedPredicates(IRequest request) 
	{
		List<String> l	= new ArrayList<String>();
		 
		for (Predicate p :  request.getMatchingPredicates())
			l.add(((BeanPredicate) p).getPredicate().getClass().getSimpleName().toString());
			 
		System.out.println("Main: logMatchingPredicates: '" + request.toString() + "', " + request.getMatchingPredicates().size() + " prédicats vérifiés: " + l.toString() + "\r\n");
	}
}
